@@ -10,12 +10,15 @@ from lmfit import Parameter
 from lmfit import (Parameters, Minimizer, conf_interval,
                    ci_report, conf_interval2d)
 
-from lmfit.minimizer import eval_stderr, MinimizerResult
+from lmfit.minimizer import MinimizerResult
 from lmfit.model import (ModelResult, save_model, load_model,
                          save_modelresult, load_modelresult)
 from lmfit.confidence import f_compare
-from uncertainties import ufloat, correlated_values
 
+from uncertainties import ufloat, correlated_values
+from uncertainties import wrap as un_wrap
+
+from ..utils import gformat, getfloat_attr
 from ..symboltable import Group, isgroup
 
 
@@ -29,6 +32,113 @@ def param_value(val):
         val = val.value
     return val
 
+def format_param(par, length=10, with_initial=True):
+    value = repr(par)
+    if isParameter(par):
+        value = gformat(par.value, length=length)
+        if not par.vary and par.expr is None:
+            value = f"{value} (fixed)"
+        else:
+            stderr = 'unknown'
+            if par.stderr is not None:
+                stderr = gformat(par.stderr, length=length)
+                value = f"{value} +/-{stderr}"
+                if par.vary and par.expr is None and with_initial:
+                    value = f"{value} (init={gformat(par.init_value, length=length)})"
+            if par.expr is not None:
+                value = f"{value}  = '{par.expr}'"
+    return value
+
+
+def stats_table(results, labels=None, csv_output=False, csv_delim=','):
+    """
+    create a table comparing fit statistics for multiple fit results
+    """
+    stats = {'number of variables': 'nvarys',
+             'chi-square': 'chi_square',
+             'reduced chi-square': 'chi2_reduced',
+             'r-factor': 'rfactor',
+             'Akaike Info Crit': 'aic',
+             'Bayesian Info Crit': 'bic'}
+
+    nfits = len(results)
+    if labels is not None:
+        if len(labels) != len(results):
+            raise ValueError('labels must be a list that is the same length as results')
+
+    columns = [['Statistics']]
+    if labels is None:
+        labels = [f"  Fit {i+1}" for i in range(nfits)]
+    for lab in labels:
+        columns.append([lab])
+
+    for sname, attr in stats.items():
+        columns[0].append(sname)
+        for i, result in enumerate(results):
+            columns[i+1].append(getfloat_attr(result, attr))
+
+    return format_table_columns(columns, csv_output=csv_output, csv_delim=csv_delim)
+
+def paramgroups_table(pgroups, labels=None, csv_output=False, csv_delim=','):
+    """
+    create a table comparing parameters from a Feffit Parameter Grooup for multiple fit results
+    """
+    nfits = len(pgroups)
+    if labels is not None:
+        if len(labels) != len(pgroups):
+            raise ValueError('labels must be a list that is the same length as Parameter Groups')
+
+    columns = [['Parameter']]
+    if labels is None:
+        labels = [f" Fit {i+1}" for i in range(nfits)]
+    for lab in labels:
+        columns.append([lab])
+
+    parnames = []
+    for pgroup in pgroups:
+        for pname in dir(pgroup):
+            if pname not in parnames:
+                parnames.append(pname)
+
+    for pname in parnames:
+        columns[0].append(pname)
+        for i, pgroup in enumerate(pgroups):
+            value = 'N/A'
+            par = getattr(pgroup, pname, None)
+            if par is not None:
+                value = format_param(par, length=10, with_initial=False)
+            columns[i+1].append(value)
+
+    return format_table_columns(columns, csv_output=csv_output, csv_delim=csv_delim)
+
+
+def format_table_columns(columns, csv_output=False, csv_delim=','):
+    hjoin, rjoin, edge = '+', '|', '|'
+    if csv_output:
+       hjoin, rjoin, edge = csv_delim, csv_delim, ''
+
+    ncols = len(columns)
+    nrows = len(columns[0])
+    slen = [2]*ncols
+    for i, col in enumerate(columns):
+        slen[i] = max(5, max([len(row) for row in col]))
+
+    buff = []
+    if not csv_output:
+        header = edge + hjoin.join(['-'*(lx+2) for lx in slen]) + edge
+        buff = [header]
+
+    while len(columns[0]) > 0:
+        values = [c.pop(0) for c in columns]
+        row = rjoin.join([f" {l:{slen[i]}.{slen[i]}s} " for i, l in enumerate(values)])
+        buff.append(edge + row + edge)
+        if not csv_output and len(buff) == 2:
+            buff.append(header)
+    if not csv_output:
+        buff.append(header)
+    return '\n'.join(buff)
+
+
 def f_test(ndata, nvars, chisquare, chisquare0, nfix=1):
     """return the F-test value for the following input values:
     f = f_test(ndata, nparams, chisquare, chisquare0, nfix=1)
@@ -43,6 +153,52 @@ def confidence_report(conf_vals, **kws):
     """
     return ci_report(conf_vals)
 
+def asteval_with_uncertainties(*vals, **kwargs):
+    """Calculate object value, given values for variables.
+
+    This is used by the uncertainties package to calculate the
+    uncertainty in an object even with a complicated expression.
+
+    """
+    _obj = kwargs.get('_obj', None)
+    _pars = kwargs.get('_pars', None)
+    _names = kwargs.get('_names', None)
+    _asteval = _pars._asteval
+    if (_obj is None or _pars is None or _names is None or
+         _asteval is None or _obj._expr_ast is None):
+        return 0
+    for val, name in zip(vals, _names):
+        _asteval.symtable[name] = val
+
+    # re-evaluate all constraint parameters to
+    # force the propagation of uncertainties
+    [p._getval() for p in _pars.values()]
+    return _asteval.eval(_obj._expr_ast)
+
+
+wrap_ueval = un_wrap(asteval_with_uncertainties)
+
+
+def eval_stderr(obj, uvars, _names, _pars):
+    """Evaluate uncertainty and set ``.stderr`` for a parameter `obj`.
+
+    Given the uncertain values `uvars` (list of `uncertainties.ufloats`),
+    a list of parameter names that matches `uvars`, and a dictionary of
+    parameter objects, keyed by name.
+
+    This uses the uncertainties package wrapped function to evaluate the
+    uncertainty for an arbitrary expression (in ``obj._expr_ast``) of
+    parameters.
+
+    """
+    if not isinstance(obj, Parameter) or getattr(obj, '_expr_ast', None) is None:
+        return
+    uval = wrap_ueval(*uvars, _obj=obj, _names=_names, _pars=_pars)
+    try:
+        obj.stderr = uval.std_dev
+    except Exception:
+        obj.stderr = 0
+
 
 class ParameterGroup(Group):
     """
@@ -353,7 +509,7 @@ def confidence_intervals(fit_result, sigmas=(1, 2, 3), **kws):
     result = getattr(fit_result, 'fit_details', None)
     return conf_interval(fitter, result, sigmas=sigmas, **kws)
 
-def chi2_map(fit_result, xname, yname, nx=11, ny=11, sigma=3, **kws):
+def chi2_map(fit_result, xname, yname, nx=21, ny=21, sigma=3, **kws):
     """generate a confidence map for any two parameters for a fit
 
     Arguments
@@ -361,8 +517,8 @@ def chi2_map(fit_result, xname, yname, nx=11, ny=11, sigma=3, **kws):
        minout   output of minimize() fit (must be run first)
        xname    name of variable parameter for x-axis
        yname    name of variable parameter for y-axis
-       nx       number of steps in x [11]
-       ny       number of steps in y [11]
+       nx       number of steps in x [21]
+       ny       number of steps in y [21]
        sigma    scale for uncertainty range [3]
 
     Returns
@@ -380,12 +536,6 @@ def chi2_map(fit_result, xname, yname, nx=11, ny=11, sigma=3, **kws):
     if fitter is None or result is None:
         raise ValueError("chi2_map needs valid fit result as first argument")
 
-    c2_scale = fit_result.chi_square / result.chisqr
-
-    def scaled_chisqr(ndata, nparas, new_chi, best_chi, nfix=1.):
-        """return scaled chi-sqaure, instead of probability"""
-        return new_chi * c2_scale
-
     x = result.params[xname]
     y = result.params[yname]
     xrange = (x.value + sigma * x.stderr, x.value - sigma * x.stderr)
@@ -393,8 +543,7 @@ def chi2_map(fit_result, xname, yname, nx=11, ny=11, sigma=3, **kws):
 
     return conf_interval2d(fitter, result, xname, yname,
                            limits=(xrange, yrange),
-                           prob_func=scaled_chisqr,
-                           nx=nx, ny=ny, **kws)
+                           nx=nx, ny=ny, nsigma=2*sigma, **kws)
 
 _larch_name = '_math'
 exports = {'param': param,
