@@ -2,15 +2,16 @@
 """
 Helper classes for larch interpreter
 """
-from __future__ import division
-import sys, os, time
+import sys
+import os
 from datetime import datetime
 import ast
-import numpy as np
 import traceback
 import toml
 import inspect
 from collections import namedtuple
+from pathlib import Path
+
 import ctypes
 import ctypes.util
 
@@ -22,18 +23,12 @@ from .utils import uname, bindir, get_cwd, read_textfile
 HAS_TERMCOLOR = False
 try:
     from termcolor import colored
-    if uname == 'win':
-        # HACK (hopefully temporary):
-        # disable color output for Windows command terminal
-        # because it interferes with wx event loop.
-        import CannotUseTermcolorOnWindowsWithWx
-        # os.environ.pop('TERM')
-        # import colorama
-        # colorama.init()
     HAS_TERMCOLOR = True
 except ImportError:
     HAS_TERMCOLOR = False
 
+if uname == 'win':
+    HAS_TERMCOLOR = False
 
 class Empty:
     def __nonzero__(self): return False
@@ -132,9 +127,9 @@ class LarchExceptionHolder:
         for tb in traceback.extract_tb(self.exc_info[2]):
             if not (sys.prefix in tb[0] and
                     ('ast.py' in tb[0] or
-                     os.path.join('larch', 'utils') in tb[0] or
-                     os.path.join('larch', 'interpreter') in tb[0] or
-                     os.path.join('larch', 'symboltable') in tb[0])):
+                     Path('larch', 'utils').as_posix() in tb[0] or
+                     Path('larch', 'interpreter').as_posix() in tb[0] or
+                     Path('larch', 'symboltable').as_posix() in tb[0])):
                 tblist.append(tb)
         if len(tblist) > 0:
             out.append(''.join(traceback.format_list(tblist)))
@@ -352,7 +347,7 @@ def add2path(envvar='PATH', dirname='.'):
         os.environ[envvar] = dirname
     else:
         paths = oldpath.split(sep)
-        paths.insert(0, os.path.abspath(dirname))
+        paths.insert(0, Path(dirname).absolute().as_posix())
         os.environ[envvar] = sep.join(paths)
     return oldpath
 
@@ -377,14 +372,14 @@ def get_dll(libname):
     # normally, we expect the dll to be here in the larch dlls tree
     # if we find it there, use that one
     fname = _dylib_formats[uname] % libname
-    dllpath = os.path.join(bindir, fname)
-    if os.path.exists(dllpath):
-        return loaddll(dllpath)
+    dllpath = Path(bindir, fname).absolute()
+    if dllpath.exists():
+        return loaddll(dllpath.as_posix())
 
     # if not found in the larch dlls tree, try your best!
-    dllpath = ctypes.util.find_library(libname)
-    if dllpath is not None and os.path.exists(dllpath):
-        return loaddll(dllpath)
+    dllpath = Path(ctypes.util.find_library(libname)).absolute()
+    if dllpath is not None and dllpath.exists():
+        return loaddll(dllpath.as_posix())
     return None
 
 
@@ -397,8 +392,8 @@ def read_workdir(conffile):
     """
 
     try:
-        w_file = os.path.join(user_larchdir, conffile)
-        if os.path.exists(w_file):
+        w_file = Path(user_larchdir, conffile).absolute()
+        if w_file.exists():
             line = open(w_file, 'r').readlines()
             workdir = line[0][:-1]
             os.chdir(workdir)
@@ -414,8 +409,8 @@ def save_workdir(conffile):
     """
 
     try:
-        w_file = os.path.join(user_larchdir, conffile)
-        fh = open(w_file, 'w')
+        w_file = Path(user_larchdir, conffile).absolute()
+        fh = open(w_file, 'w', encoding=sys.getdefaultencoding())
         fh.write("%s\n" % get_cwd())
         fh.close()
     except:
@@ -428,9 +423,9 @@ def read_config(conffile):
 
     returns dictionary / configuration
     """
-    cfile = os.path.join(user_larchdir, conffile)
+    cfile = Path(user_larchdir, conffile).absolute()
     out = None
-    if os.path.exists(cfile):
+    if cfile.exists():
         data = read_textfile(cfile)
         try:
             out = toml.loads(data)
@@ -443,7 +438,7 @@ def save_config(conffile, config):
     compare read_confif(conffile) which will read this value
 
     """
-    cfile = os.path.join(user_larchdir, conffile)
+    cfile = Path(user_larchdir, conffile).absolute()
     dat = toml.dumps(config).encode('utf-8')
     with open(cfile, 'wb') as fh:
         fh.write(dat)
