@@ -1,31 +1,27 @@
 #!/usr/bin/env python
 """Version information"""
-__date__    = '2023-Mar-06'
-__release_version__ = '0.9.67'
-__authors__ = "M. Newville, M. Koker, M. Rovezzi, B. Ravel, and others"
+
+__release_version__ = '2025.2.2'
+__date__    = '2025-July-29'
+__authors__ = "M. Newville, M. Rovezzi, M. Koker, B. Ravel, and others"
+
+from ._version import __version__, __version_tuple__
 import sys
 from collections import namedtuple
 from packaging.version import parse as ver_parse
+
 import importlib
 import urllib3
-import requests
-import numpy
-import scipy
-import matplotlib
-import lmfit
-
-try:
-    from importlib.metadata import version, PackageNotFoundError
-except ImportError:
-    from importlib_metadata import version, PackageNotFoundError
-try:
-    __version__ = version("xraylarch")
-except PackageNotFoundError:
-    # package is not installed
-    __version__ = __release_version__
+from subprocess import check_call
 
 urllib3.disable_warnings(urllib3.exceptions.InsecureRequestWarning)
 
+# libraries whose versions might be interesting to know
+LIBS_VERSIONS = ('numpy', 'scipy', 'matplotlib', 'h5py', 'sklearn', 'skimage',
+                 'sqlalchemy', 'fabio', 'pyFAI', 'PIL', 'imageio', 'silx',
+                 'tomopy', 'pymatgen.core', 'numdifftools', 'xraydb', 'lmfit',
+                 'asteval', 'wx', 'wxmplot')
+
 def version_data(with_libraries=False):
     "get version data"
     vinf  = sys.version_info
@@ -53,11 +49,7 @@ def version_data(with_libraries=False):
             }
 
     if with_libraries:
-        for modname in ('numpy', 'scipy', 'matplotlib', 'h5py', 'sklearn',
-                        'skimage', 'sqlalchemy', 'fabio', 'pyFAI', 'PIL',
-                        'imageio', 'silx', 'tomopy', 'pymatgen.core',
-                        'numdifftools', 'xraydb', 'lmfit', 'asteval', 'wx',
-                        'wxmplot'):
+        for modname in LIBS_VERSIONS:
             vers = "not installed"
             if modname not in sys.modules:
                 try:
@@ -73,16 +65,31 @@ def version_data(with_libraries=False):
             vdat[modname] = vers
     return vdat
 
-def make_banner(mods=None):
+def make_banner(with_libraries=False, show_libraries=None):
     "return startup banner"
-    vdat = version_data()
+    if show_libraries is None:
+        show_libraries = LIBS_VERSIONS if with_libraries else []
+
+    vdat = version_data(with_libraries=True)
     lines = [f"Larch {vdat['release version']}, released {vdat['release date']}"]
     if vdat['development version'] != vdat['release version']:
         lines.append(f"development version: {vdat['development version']}")
     lines.append(f"Python {vdat['python version']}, {vdat['python builder']}")
-    lines.append('use `print(show_version())` for version details')
+    libline = []
+    for key, val in vdat.items():
+        if key in show_libraries:
+            libline.append(f"{key:s}: {val}")
+            if len(libline) > 3:
+                lines.append(', '.join(libline))
+                libline = []
+    if len(libline) > 0:
+        lines.append(', '.join(libline))
+
+    if len(show_libraries) < 10:
+        lines.append('use `print(show_version())` for detailed versions')
+
     linelen = max([len(line) for line in lines])
-    border = '='*min(75, max(linelen, 25))
+    border = '='*min(99, max(linelen, 25))
     lines.insert(0, border)
     lines.append(border)
     return '\n'.join(lines)
@@ -117,7 +124,7 @@ LATEST_MESSAGE = """Larch version {local_version:s} is up to date."""
 def check_larchversion():
     "check version, return VersionStatus tuple"
     local_version = __release_version__
-
+    import requests
     try:
         req = requests.get(VERSION_URL, verify=False, timeout=3.10)
     except:
@@ -137,3 +144,26 @@ def check_larchversion():
     message = message.format(remote_version=remote_version,
                              local_version=local_version)
     return VersionStatus(update_available,local_version, remote_version, message)
+
+NIGHTLY_URL  = 'https://millenia.cars.aps.anl.gov/xraylarch/downloads/'
+NIGHTLY_WHEEL = 'xraylarch-latest.txt'
+
+def upgrade_nightly_build():
+    "pip install from latest nightly build"
+    import requests
+    try:
+        url = f'{NIGHTLY_URL}/{NIGHTLY_WHEEL}'
+        req = requests.get(f'{NIGHTLY_URL}/{NIGHTLY_WHEEL}', verify=False, timeout=10.0)
+    except:
+        print("could not get nightly build", url)
+        return
+
+    if req.status_code == 200:
+        fname = req.text.split()[0]
+        print("Downloading wheel for nightly build: {fname}")
+        check_call([sys.executable, '-m', 'pip', 'install', f'{NIGHTLY_URL}{fname}'])
+
+def upgrade_from_pypi():
+    "pip installl/upgrade larch from PyPI"
+    target = 'xraylarch[larix]' if with_larix else 'xraylarch'
+    check_call([sys.executable, '-m', 'pip', 'install', '--upgrade', target])
