@@ -13,8 +13,9 @@ fileprompt       launch file browser to select files.
 '''
 
 import locale
+from pathlib import Path
 
-from pyshortcuts import platform
+from pyshortcuts import uname, fix_filename
 import os
 import sys
 HAS_WXPYTHON = False
@@ -27,59 +28,100 @@ except (ImportError, AttributeError):
 _larch_name = '_sys.wx'
 _larch_builtins = {}
 
-FONTSIZE = 11
-FONTSIZE_FW = 11
-if platform == 'win':
+FONTSIZE = 10
+FONTSIZE_FW = 10
+if uname == 'win':
     FONTSIZE = 10
-    FONTSIZE_FW = 10
+    FONTSIZE_FW = 11
     locale.setlocale(locale.LC_ALL, 'C')
+elif uname == 'darwin':
+    FONTSIZE = 11
+    FONTSIZE_FW = 12
+
+
+def DarwinHLine(parent, size=(700, 3)):
+    """Horizontal line for MacOS
+    h = HLine(parent, size=(700, 3)
+    """
+    msize = (size[0], int(size[1]*0.75))
+    line = wx.Panel(parent, size=msize)
+    line.SetBackgroundColour((196,196,196))
+    return line
+
+DARK_THEME = False
+try:
+    import darkdetect
+    DARK_THEME = darkdetect.isDark()
+except ImportError:
+    DARK_THEME = False
+
+def nullfunc(*args, **kws):
+    pass
+
+_larch_builtins = {'_sys.wx': dict(gcd=nullfunc,
+                                   databrowser=nullfunc,
+                                   filepromspt=nullfunc,
+                                   wx_update=nullfunc)}
+
+_larch_builtins['_plotter'] = dict(plot=nullfunc,
+                                   oplot=nullfunc,
+                                   newplot=nullfunc,
+                                   plot_text=nullfunc,
+                                   plot_marker=nullfunc,
+                                   plot_arrow=nullfunc,
+                                   plot_setlimits=nullfunc,
+                                   plot_axvline=nullfunc,
+                                   plot_axhline=nullfunc,
+                                   scatterplot=nullfunc,
+                                   hist=nullfunc,
+                                   update_trace=nullfunc,
+                                   save_plot=nullfunc,
+                                   save_image=nullfunc,
+                                   get_display=nullfunc,
+                                   close_all_displays=nullfunc,
+                                   get_cursor=nullfunc,
+                                   last_cursor_pos=nullfunc,
+                                   imshow=nullfunc,
+                                   contour=nullfunc,
+                                   xrf_plot=nullfunc,
+                                   xrf_oplot=nullfunc,
+                                   fit_plot=nullfunc)
 
 if HAS_WXPYTHON:
-    try:
-        import wxutils
-        import wxmplot
-    except:
-        from ..apps import install_extras
-        from ..site_config import extras_wxgraph
-        install_extras(extras_wxgraph)
-
+    from wxutils import (set_sizer, pack, SetTip, Font, HLine, Check,
+                         MenuItem, Popup, RIGHT, LEFT, CEN , LTEXT,
+                         FRAMESTYLE, hms, DateTimeCtrl, Button,
+                         TextCtrl, ToggleButton, BitmapButton, Choice,
+                         YesNo, SimpleText, LabeledTextCtrl,
+                         HyperText, get_icon, OkCancel,
+                         SavedParameterDialog, GridPanel, RowPanel,
+                         make_steps, set_float, FloatCtrl,
+                         EditableListBox, FileCheckList,
+                         FileDropTarget, NumericCombo, FloatSpin,
+                         FileOpen, FileSave, SelectWorkdir,
+                         FloatSpinWithPin, flatnotebook,
+                         PeriodicTablePanel, gcd, ExceptionPopup,
+                         show_wxsizes, panel_pack)
 
+    from .wxcolors import COLORS, GUIColors, GUI_COLORS, set_color
     from . import larchframe
     from . import larchfilling
     from . import readlinetextctrl
 
-    from wxutils import (set_sizer, pack, SetTip, Font, HLine, Check,
-                         MenuItem, Popup, RIGHT, LEFT, CEN , LTEXT,
-                         FRAMESTYLE, hms, DateTimeCtrl, Button, TextCtrl,
-                         ToggleButton, BitmapButton, Choice, YesNo,
-                         SimpleText, LabeledTextCtrl, HyperText, get_icon,
-                         OkCancel, FileOpen, FileSave, SelectWorkdir,
-                         fix_filename, SavedParameterDialog, GridPanel,
-                         RowPanel, make_steps, set_float, FloatCtrl,
-                         EditableListBox)
-
-    from .colors import COLORS, GUIColors, set_color
-    from .filechecklist import FileCheckList, FileDropTarget
-
-    from .floats import NumericCombo, FloatSpin, FloatSpinWithPin
-
-    from .notebooks import flatnotebook
-
     from .larchframe import LarchFrame, LarchPanel
     from .columnframe import ColumnDataFileFrame, EditColumnFrame
     from .athena_importer import AthenaImporter
     from .specfile_importer import SpecfileImporter
-    from .reportframe import ReportFrame, DictFrame, DataTableGrid
-    from .gui_utils import (gcd, databrowser, fileprompt, LarchWxApp,
-                            wx_update, show_wxsizes, panel_pack)
-
+    from .xas_importer import XasImporter
+    from .reportframe import ReportFrame, DictFrame, DataTableGrid, CSVFrame
+    from .gui_utils import (databrowser, fileprompt, LarchWxApp, wx_update)
     from .larch_updater import LarchUpdaterDialog
     from .parameter import ParameterWidgets, ParameterPanel
-    from .periodictable import PeriodicTablePanel
+    from .xafsplots import plotlabels
+
     from .feff_browser import FeffResultsFrame, FeffResultsPanel
-
     from .cif_browser import CIFFrame
-
+    from .structure2feff_browser import Structure2FeffFrame
 
     _larch_builtins = {'_sys.wx': dict(gcd=gcd,
                                        databrowser=databrowser,
@@ -95,10 +137,10 @@ if HAS_WXPYTHON:
                           _xrf_oplot, _fitplot, _redraw_plot,
                           get_zoomlimits, set_zoomlimits)
 
-    # from .xrfdisplay import  XRFDisplayFrame
 
-    from . import xafsplots
-    from .xafsplots import plotlabels
+
+    if uname == 'darwin':
+        HLine = DarwinHLine
 
     _larch_builtins['_plotter'] = dict(plot=_plot, oplot=_oplot,
                                        newplot=_newplot, plot_text=_plot_text,
@@ -106,7 +148,7 @@ if HAS_WXPYTHON:
                                        plot_arrow=_plot_arrow,
                                        plot_setlimits=_plot_setlimits,
                                        plot_axvline=_plot_axvline,
-                                       plot_axhline=_plot_axhline,
+                                      plot_axhline=_plot_axhline,
                                        scatterplot=_scatterplot, hist=_hist,
                                        update_trace=_update_trace,
                                        save_plot=_saveplot,
@@ -123,7 +165,7 @@ if HAS_WXPYTHON:
                                        redraw_plot=_redraw_plot)
 
     _larch_builtins['_xafs'] = dict(redraw=xafsplots.redraw,
-                                    plotlabels=xafsplots.plotlabels,
+                                    plotlabels=plotlabels,
                                     plot_mu=xafsplots.plot_mu,
                                     plot_bkg=xafsplots.plot_bkg,
                                     plot_chie=xafsplots.plot_chie,
@@ -142,7 +184,9 @@ if HAS_WXPYTHON:
                                     plot_prepeaks_baseline=xafsplots.plot_prepeaks_baseline,
                                     plot_pca_components=xafsplots.plot_pca_components,
                                     plot_pca_weights=xafsplots.plot_pca_weights,
-                                    plot_pca_fit=xafsplots.plot_pca_fit)
+                                    plot_pca_fit=xafsplots.plot_pca_fit,
+                                    plot_curvefit=xafsplots.plot_curvefit,
+                                    )
 
 
     def _larch_init(_larch):
@@ -154,35 +198,27 @@ if HAS_WXPYTHON:
             _sys.gui_apps = {}
         # _sys.gui_apps['xrfviewer'] = ('XRF Spectrum Viewer', XRFDisplayFrame)
 
-else:
-    def nullfunc(*args, **kws):
-        pass
+    #############################
+    ## Hack System and Startfile on Windows totry to track down
+    ## weird error of starting other applications, like Mail
+    if uname == 'win':
+        from os import system as os_system
+        from os import startfile as os_startfile
 
-    _larch_builtins = {'_sys.wx': dict(gcd=nullfunc,
-                                       databrowser=nullfunc,
-                                       filepromspt=nullfunc,
-                                       wx_update=nullfunc)}
+        def my_system(command):
+            print(f"#@-> os.system: {command}")
+            return os_system(command)
 
-    _larch_builtins['_plotter'] = dict(plot=nullfunc,
-                                       oplot=nullfunc,
-                                       newplot=nullfunc,
-                                       plot_text=nullfunc,
-                                       plot_marker=nullfunc,
-                                       plot_arrow=nullfunc,
-                                       plot_setlimits=nullfunc,
-                                       plot_axvline=nullfunc,
-                                       plot_axhline=nullfunc,
-                                       scatterplot=nullfunc,
-                                       hist=nullfunc,
-                                       update_trace=nullfunc,
-                                       save_plot=nullfunc,
-                                       save_image=nullfunc,
-                                       get_display=nullfunc,
-                                       close_all_displays=nullfunc,
-                                       get_cursor=nullfunc,
-                                       last_cursor_pos=nullfunc,
-                                       imshow=nullfunc,
-                                       contour=nullfunc,
-                                       xrf_plot=nullfunc,
-                                       xrf_oplot=nullfunc,
-                                       fit_plot=nullfunc)
+        def my_startfile(filepath, operation=None):
+            print(f"#@-> os.startfile: {filepath}, {operation}")
+            try:
+                if operation is None:
+                    return os_startfile(filepath)
+                else:
+                    return os_startfile(filepath, operation)
+            except WindowsError:
+                print(f"#@-> os.startfile failed: {filepath}, {operation}")
+
+        os.system = my_system
+        os.startfile = my_startfile
+    #############################
