@@ -4,23 +4,26 @@ import time
 import logging
 import shutil
 from datetime import datetime, timedelta
+from pathlib import Path
+
 import wx
 import wx.lib.scrolledpanel as scrolled
 import wx.dataview as dv
 
 import larch
 from larch.site_config import user_larchdir
-from larch.utils import unixpath, read_textfile
+from larch.utils import unixpath, mkdir, read_textfile
 from larch.wxlib import (GridPanel, GUIColors, Button, pack, SimpleText,
-                         FileOpen, FileSave, Font, LEFT, FRAMESTYLE,
+                         Font, LEFT, FRAMESTYLE,
                          FONTSIZE, MenuItem, EditableListBox, OkCancel,
                          FileCheckList, Choice, HLine, ReportFrame, Popup,
                          LarchWxApp)
+from .wxcolors import GUI_COLORS
 
 from larch.xafs import get_feff_pathinfo
-from larch.xray import atomic_symbols
+from larch.utils.physical_constants import ATOM_SYMS
 
-ATSYMS = ['< All Atoms>'] + atomic_symbols
+ATSYMS = ['< All Atoms>'] + ATOM_SYMS[:96]
 EDGES  = ['< All Edges>', 'K', 'L3', 'L2', 'L1', 'M5']
 
 
@@ -102,19 +105,19 @@ class FeffPathsModel(dv.DataViewIndexListModel):
         nleg = self.data[row][2]
         cname = self.data[row][0]
         if nleg == '2':
-            attr.SetColour('#000')
+            attr.SetColour(GUI_COLORS.text)
             attr.SetBold(False)
             return True
         elif nleg == '3':
-            attr.SetColour('#A11')
+            attr.SetColour(GUI_COLORS.title_red)
             attr.SetBold(False)
             return True
         elif nleg == '4':
-            attr.SetColour('#11A')
+            attr.SetColour(GUI_COLORS.title_blue)
             attr.SetBold(False)
             return True
         else:
-            attr.SetColour('#393')
+            attr.SetColour(GUI_COLORS.title_green)
             attr.SetBold(False)
             return True
         return False
@@ -143,18 +146,17 @@ class RemoveFeffCalcDialog(wx.Dialog):
 
 class FeffResultsPanel(wx.Panel):
     """ present Feff results """
-    def __init__(self, parent=None, feffresult=None, path_importer=None, xasmain=None,
+    def __init__(self, parent=None, feffresult=None, path_importer=None,
                  _larch=None):
-        wx.Panel.__init__(self, parent, -1, size=(850, 650))
+        wx.Panel.__init__(self, parent, -1, size=(700, 500))
         self.parent = parent
-        self.xasmain = xasmain
         self.path_importer = path_importer
         self._larch = _larch
         self.feffresult = feffresult
         self.report_frame = None
 
         self.dvc = dv.DataViewCtrl(self, style=DVSTYLE)
-        self.dvc.SetMinSize((825, 350))
+        self.dvc.SetMinSize((695, 350))
 
         self.model = FeffPathsModel(None, with_use=callable(path_importer))
         self.dvc.AssociateModel(self.model)
@@ -227,14 +229,14 @@ class FeffResultsPanel(wx.Panel):
 
         pack(panel, sizer)
 
-        columns = [('Feff File',   125, 'text'),
-                   ('R (\u212B)',   75, 'text'),
-                   ('# legs',       55, 'text'),
-                   ('# paths',      55, 'text'),
+        columns = [('Feff File',   100, 'text'),
+                   ('R (\u212B)',   60, 'text'),
+                   ('# legs',       60, 'text'),
+                   ('# paths',      65, 'text'),
                    ('Importance',  100, 'text')]
         if callable(self.path_importer):
             columns.append(('Use',     50, 'bool'))
-        columns.append(('Geometry',   175, 'text'))
+        columns.append(('Geometry',   200, 'text'))
 
         for icol, dat in enumerate(columns):
              label, width, dtype = dat
@@ -245,14 +247,16 @@ class FeffResultsPanel(wx.Panel):
                  mode = dv.DATAVIEW_CELL_ACTIVATABLE
              method(label, icol, width=width, mode=mode)
              c = self.dvc.Columns[icol]
-             align = wx.ALIGN_RIGHT if icol in (1, 2, 3, 4) else wx.ALIGN_LEFT
+             align = wx.ALIGN_RIGHT
+             if (label.startswith('Feff') or label.startswith('Geom')):
+                 align = wx.ALIGN_LEFT
              c.Alignment = c.Renderer.Alignment = align
              c.SetSortable(False)
 
 
         mainsizer = wx.BoxSizer(wx.VERTICAL)
-        mainsizer.Add(panel,    0, LEFT|wx.GROW, 1)
-        mainsizer.Add(self.dvc, 1, LEFT|wx.GROW, 1)
+        mainsizer.Add(panel,    0, LEFT, 1)
+        mainsizer.Add(self.dvc, 0, LEFT, 1)
 
         pack(self, mainsizer)
         self.dvc.EnsureVisible(self.model.GetItem(0))
@@ -305,13 +309,13 @@ class FeffResultsPanel(wx.Panel):
     def onShowFeffInp(self, event=None):
         if self.feffresult is not None:
             text = None
-            fname = unixpath(os.path.join(self.feffresult.folder, 'feff.inp'))
-            if os.path.exists(fname):
+            fname = Path(self.feffresult.folder, 'feff.inp')
+            if fname.exists():
                 text = read_textfile(fname)
             else:
-                fname = unixpath(os.path.join(user_larchdir, 'feff',
-                                              self.feffresult.folder, 'feff.inp'))
-                if os.path.exists(fname):
+                fname = Path(user_larchdir, 'feff',
+                                 self.feffresult.folder, 'feff.inp')
+                if fname.exists():
                     text = read_textfile(fname)
             if text is not None:
                 self.show_report(text, title=f'Feff.inp for {self.feffresult.folder:s}',
@@ -321,7 +325,7 @@ class FeffResultsPanel(wx.Panel):
     def show_report(self, text, title='Text', default_filename='out.txt', wildcard=None):
         if wildcard is None:
             wildcard='Text Files (*.txt)|*.txt'
-        default_filename = os.path.split(default_filename)[1]
+        default_filename = Path(default_filename).name
         try:
             self.report_frame.set_text(text)
             self.report_frame.SetTitle(title)
@@ -336,11 +340,11 @@ class FeffResultsPanel(wx.Panel):
 
     def onImportPath(self, event=None):
         folder = self.feffresult.folder
-        _, fname = os.path.split(folder)
+        fname = Path(folder).name
         for data in self.model.data:
             if data[5]:
                 fname = data[0]
-                fullpath = unixpath(os.path.join(folder, fname))
+                fullpath = Path(folder, fname).as_posix()
                 for pathinfo in self.feffresult.paths:
                     if pathinfo.filename == fname:
                         self.path_importer(fullpath, pathinfo)
@@ -367,9 +371,8 @@ class FeffResultsPanel(wx.Panel):
 
 class FeffResultsFrame(wx.Frame):
     """ present Feff results """
-    def __init__(self,  parent=None, feffresult=None, xasmain=None,
-                 path_importer=None, _larch=None):
-        wx.Frame.__init__(self, parent, -1, size=(850, 650), style=FRAMESTYLE)
+    def __init__(self,  parent=None, feffresult=None, path_importer=None, _larch=None):
+        wx.Frame.__init__(self, parent, -1, size=(900, 650), style=FRAMESTYLE)
 
         title = "Manage Feff calculation results"
         self.larch = _larch
@@ -381,10 +384,8 @@ class FeffResultsFrame(wx.Frame):
             self.larch.symtable._sys._feffruns = {}
         self.parent = parent
 
-        path = unixpath(os.path.join(user_larchdir, 'feff'))
-        if not os.path.exists(path):
-            os.makedirs(path, mode=493)
-        self.feff_folder = path
+        self.feff_folder = unixpath(Path(user_larchdir, 'feff'))
+        mkdir(self.feff_folder)
 
         self.SetTitle(title)
         self.SetSize((925, 650))
@@ -435,9 +436,9 @@ class FeffResultsFrame(wx.Frame):
         wids['edge']         = Choice(toprow, choices=EDGES, size=(125, -1),
                                       action=self.onAbsorbingEdge)
 
-        flabel = SimpleText(toprow, 'Filter Calculations by Element and Edge:', size=(200, -1))
+        flabel = SimpleText(toprow, 'Filter Calculations by Element and Edge:', size=(175, -1))
         tsizer = wx.BoxSizer(wx.HORIZONTAL)
-        tsizer.Add(flabel,               1, LEFT, 2)
+        tsizer.Add(flabel,               0, LEFT, 2)
         tsizer.Add(wids['central_atom'], 0, LEFT|wx.GROW, 2)
         tsizer.Add(wids['edge'],         0, LEFT|wx.GROW, 2)
         pack(toprow, tsizer)
@@ -446,7 +447,7 @@ class FeffResultsFrame(wx.Frame):
         self.feff_panel = FeffResultsPanel(panel, path_importer=path_importer,
                                            _larch=_larch)
         sizer.Add(toprow, 0, LEFT|wx.GROW|wx.ALL, 2)
-        sizer.Add(HLine(panel, size=(550, 2)), 0,  LEFT|wx.GROW|wx.ALL, 2)
+        sizer.Add(HLine(panel, size=(650, 2)), 0,  LEFT|wx.GROW|wx.ALL, 2)
         sizer.Add(self.feff_panel, 1, LEFT|wx.GROW|wx.ALL, 2)
         pack(panel, sizer)
         # panel.SetupScrolling()
@@ -469,11 +470,11 @@ class FeffResultsFrame(wx.Frame):
         self.fefflist.Clear()
         self.feffruns = {}
         flist = os.listdir(self.feff_folder)
-        flist = sorted(flist, key=lambda t: -os.stat(unixpath(os.path.join(self.feff_folder, t))).st_mtime)
+        flist = sorted(flist, key=lambda t: -os.stat(Path(self.feff_folder, t)).st_mtime)
         _feffruns = self.larch.symtable._sys._feffruns
         for path in flist:
-            fullpath = unixpath(os.path.join(self.feff_folder, path))
-            if os.path.isdir(fullpath):
+            fullpath = Path(self.feff_folder, path)
+            if fullpath.is_dir():
                 try:
                     _feffruns[path] = thisrun = get_feff_pathinfo(fullpath)
                     if ((len(thisrun.paths) < 1) or
@@ -508,7 +509,7 @@ class FeffResultsFrame(wx.Frame):
         dlg.Destroy()
         if remove:
             for checked in self.fefflist.GetCheckedStrings():
-                shutil.rmtree(unixpath(os.path.join(self.feff_folder, checked)))
+                shutil.rmtree(unixpath(Pathn(self.feff_folder, checked)))
             self.onSearch()
 
     def onFeffFolder(self, event=None):
@@ -519,10 +520,8 @@ class FeffResultsFrame(wx.Frame):
         dlg.SetPath(self.feff_folder)
         if  dlg.ShowModal() == wx.ID_CANCEL:
             return None
-        path = os.path.abspath(dlg.GetPath())
-        if not os.path.exists(path):
-            os.makedirs(path, mode=493)
-        self.feff_folder = path
+        self.feff_folder = Path(dlg.GetPath()).absolute()
+        Path.mkdir(self.feff_folder, mode=755, parents=True, exist_ok=True)
 
     def onImportFeffCalc(self, event=None):
         "prompt to import Feff calculation folder"
@@ -532,13 +531,13 @@ class FeffResultsFrame(wx.Frame):
         dlg.SetPath(self.feff_folder)
         if  dlg.ShowModal() == wx.ID_CANCEL:
             return None
-        path = os.path.abspath(dlg.GetPath())
-        if os.path.exists(path):
+        path = Path(dlg.GetPath()).absolute()
+        if path.exists():
             flist = os.listdir(path)
             if ('paths.dat' in flist and 'files.dat' in flist and
                 'feff0001.dat' in flist and 'feff.inp' in flist):
-                _, dname = os.path.split(path)
-                dest = unixpath(os.path.join(self.feff_folder, dname))
+                dname = Path(path).name
+                dest = unixpath(Path(self.feff_folder, dname))
                 shutil.copytree(path, dest)
                 self.onSearch()
             else:
