@@ -4,11 +4,10 @@ GUI Frame for XRF display, reading larch MCA group
 
 """
 import sys
-import os
 import time
 import copy
 from functools import partial
-
+from pathlib import Path
 import wx
 import wx.lib.mixins.inspection
 import wx.lib.scrolledpanel as scrolled
@@ -19,22 +18,23 @@ import numpy as np
 import matplotlib
 from matplotlib.ticker import LogFormatter, FuncFormatter
 
+from pyshortcuts import uname, bytes2str, get_cwd, fix_filename
+
 from wxmplot import PlotPanel
-from wxutils import (SimpleText, EditableListBox, Font, pack, Popup,
-                     get_icon, SetTip, Button, Check, MenuItem, Choice,
-                     FileOpen, FileSave, fix_filename, HLine, GridPanel,
-                     CEN, LEFT, RIGHT)
-from pyshortcuts import platform
-from . import FONTSIZE, FONTSIZE_FW
+from . import (SimpleText, EditableListBox, Font, pack, Popup,
+               get_icon, SetTip, Button, Check, MenuItem, Choice,
+               FileOpen, FileSave, HLine, GridPanel,
+               CEN, LEFT, RIGHT, PeriodicTablePanel,
+               FONTSIZE, FONTSIZE_FW)
+
 from ..math import index_of
-from ..utils import bytes2str, debugtime, get_cwd
 from ..io import GSEMCA_File
 from ..site_config import icondir
 from ..interpreter import Interpreter
 
 from .gui_utils import LarchWxApp
 from .larchframe import LarchFrame
-from .periodictable import PeriodicTablePanel
+# from .periodictable import PeriodicTablePanel
 
 from .xrfdisplay_utils import (XRFCalibrationFrame, ColorsFrame,
                                XrayLinesFrame, XRFDisplayConfig, XRFGROUP,
@@ -132,13 +132,16 @@ class XRFDisplayFrame(wx.Frame):
         self._menus = []
         self.createMainPanel()
         self.createMenus()
-        self.SetFont(Font(9, serif=True))
+        self.SetFont(wx.Font(9, wx.SWISS, wx.NORMAL, wx.NORMAL))
         self.statusbar = self.CreateStatusBar(4)
         self.statusbar.SetStatusWidths([-5, -3, -3, -4])
         statusbar_fields = ["XRF Display", " ", " ", " "]
         for i in range(len(statusbar_fields)):
             self.statusbar.SetStatusText(statusbar_fields[i], i)
         if filename is not None:
+            if isinstance(filename, Path):
+                filename = Path(filename).absolute().as_posix()
+
             self.add_mca(GSEMCA_File(filename), filename=filename, plot=True)
 
 
@@ -289,7 +292,7 @@ class XRFDisplayFrame(wx.Frame):
 
     def createControlPanel(self):
         ctrlpanel = wx.Panel(self, name='Ctrl Panel')
-        ptable_fontsize = 11 if platform=='darwin' else 9
+        ptable_fontsize = 11 if uname=='darwin' else 9
         ptable = PeriodicTablePanel(ctrlpanel, onselect=self.onShowLines,
                                     tooltip_msg='Select Element for KLM Lines',
                                     fontsize=ptable_fontsize, size=(360, 180))
@@ -411,13 +414,15 @@ class XRFDisplayFrame(wx.Frame):
         xlines.SetFont(self.font_fixedwidth)
         self.wids['xray_lines'] = xlines
 
-        xw = (60, 100, 80, 100)
-        if platform=='win':
-            xw = (65, 105, 85, 100)
-        xlines.AppendTextColumn(' Line ',         width=xw[0])
-        xlines.AppendTextColumn(' Energy(keV) ',  width=xw[1])
-        xlines.AppendTextColumn(' Strength ',     width=xw[2])
-        xlines.AppendTextColumn(' Levels ',       width=xw[3])
+        xw = (55, 105, 90, 95)
+        if uname == 'darwin':
+            xw = (55, 80, 65, 120)
+        elif uname == 'win':
+            xw = (55, 120, 90, 90)
+        xlines.AppendTextColumn('Line',         width=xw[0])
+        xlines.AppendTextColumn('Energy(keV)',  width=xw[1])
+        xlines.AppendTextColumn('Strength',     width=xw[2])
+        xlines.AppendTextColumn('Levels',       width=xw[3])
         for col in (0, 1, 2, 3):
             this = xlines.Columns[col]
             this.Sortable = False
@@ -428,7 +433,8 @@ class XRFDisplayFrame(wx.Frame):
 
         xlines.SetMinSize((300, 240))
         xlines.Bind(dv.EVT_DATAVIEW_SELECTION_CHANGED,
-                    self.onSelectXrayLine)
+
+        self.onSelectXrayLine)
         store = xlines.GetStore()
 
         # main layout
@@ -479,7 +485,7 @@ class XRFDisplayFrame(wx.Frame):
         if not symtab.has_group(XRFGROUP):
             self.larch.eval(MAKE_XRFGROUP_CMD)
 
-        fico = os.path.join(icondir, ICON_FILE)
+        fico = Path(icondir, ICON_FILE).as_posix()
         try:
             self.SetIcon(wx.Icon(fico, wx.BITMAP_TYPE_ICO))
         except:
@@ -495,13 +501,15 @@ class XRFDisplayFrame(wx.Frame):
         xrfgroup = self.larch.symtable.get_group(XRFGROUP)
         mcaname = next_mcaname(self.larch)
         if filename is not None:
+            if isinstance(filename, Path):
+                filename = Path(filename).absolute().as_posix()
             self.larch.eval(read_mcafile.format(group=XRFGROUP,
                                                 name=mcaname,
                                                 filename=filename))
             if label is None:
-                label = filename
+                label = Path(filename).absolute().name
         if label is None and hasattr(mca, 'filename'):
-            label = mca.filename
+            label = Path(mca.filename).absolute().name
         if label is None:
             label = mcaname
         self.mca.label = label
@@ -1174,7 +1182,8 @@ class XRFDisplayFrame(wx.Frame):
 
         self.xdata = 1.0*x[:]
         self.ydata = 1.0*y[:]
-        ydat = 1.0*y[:] + 1.e-9
+        self.ydata[np.where(self.ydata<1.e-9)] = 1.e-9
+        ydat = self.ydata
         kwargs['ymax'] = max(ydat)*1.25
         kwargs['ymin'] = 0.9
         kwargs['xmax'] = max(self.xdata)
@@ -1195,9 +1204,11 @@ class XRFDisplayFrame(wx.Frame):
                     continue
                 yroi[r.left:r.right] = y[r.left:r.right]
             yroi = np.ma.masked_less(yroi, 0)
+            xroi = 1.0*x[:]
+            xroi[yroi< 0.0] = np.nan
             if yroi.max() > 0:
                 kwargs['color'] = self.conf.roi_color
-                panel.oplot(x, yroi, label='rois', **kwargs)
+                panel.oplot(xroi, yroi, label='rois', **kwargs)
         yscale = {False:'linear', True:'log'}[self.ylog_scale]
         panel.set_viewlimits()
         panel.set_logscale(yscale=yscale)
@@ -1215,9 +1226,9 @@ class XRFDisplayFrame(wx.Frame):
         if is_mca2:
             mca = self.mca2
             ix = 2
-        mca.counts = counts[:]
+        mca.counts = 1.0*counts[:]
         if energy is not None:
-            mca.energy = energy[:]
+            mca.energy = 1.0*energy[:]
         xnpts = 1.0/len(energy)
         nrois = len(mca.rois)
         if not is_mca2 and with_rois and nrois > 0:
@@ -1289,7 +1300,7 @@ class XRFDisplayFrame(wx.Frame):
 
         filename = None
         if dlg.ShowModal() == wx.ID_OK:
-            filename = os.path.abspath(dlg.GetPath())
+            filename = Path(dlg.GetPath()).absolute().as_posix()
         dlg.Destroy()
 
         if filename is None:
@@ -1312,8 +1323,7 @@ class XRFDisplayFrame(wx.Frame):
         if not deffile.endswith('.mca'):
             deffile = deffile + '.mca'
 
-        _, deffile = os.path.split(deffile)
-        deffile = fix_filename(str(deffile))
+        deffile = fix_filename(Path(deffile).name)
         outfile = FileSave(self, "Save MCA File",
                            default_file=deffile,
                            wildcard=FILE_WILDCARDS)
@@ -1335,8 +1345,7 @@ class XRFDisplayFrame(wx.Frame):
         if not deffile.endswith('.dat'):
             deffile = deffile + '.dat'
 
-        _, deffile = os.path.split(deffile)
-        deffile = fix_filename(str(deffile))
+        deffile = fix_filename(Path(deffile).name)
         ASCII_WILDCARDS = "Data File (*.dat)|*.dat|All files (*.*)|*.*"
         outfile = FileSave(self, "Save ASCII File for MCA Data",
                            default_file=deffile,
@@ -1392,12 +1401,8 @@ class XRFDisplayFrame(wx.Frame):
                 read = (wx.ID_YES == Popup(self, "Re-read file '%s'?" % path,
                                            'Re-read file?', style=wx.YES_NO))
         dlg.Destroy()
+        return path
 
-        if read:
-            try:
-                parent, fname = os.path.split(path)
-            except:
-                return
 
 class XRFApp(LarchWxApp):
     def __init__(self, filename=None, **kws):
