@@ -1,26 +1,30 @@
 import os
-import copy
 import time
 import shutil
 from glob import glob
-import numpy as np
+from pathlib import Path
 from copy import deepcopy
 
+import numpy as np
 import wx
+import darkdetect
 
+from pyshortcuts import fix_varname, get_cwd
 import larch
+from larch import Group, Journal, Entry
 from larch.larchlib import read_config, save_config
-from larch.utils import (group2dict, unique_name, fix_varname, get_cwd,
-                         asfloat, get_sessionid)
+from larch.utils import (group2dict, unique_name,
+                         asfloat, get_sessionid, mkdir, unixpath)
 from larch.wxlib.plotter import last_cursor_pos
-from larch.io import fix_varname, save_session
+from larch.wxlib import ExceptionPopup
+from larch.io import save_session
 from larch.site_config import home_dir, user_larchdir
 
-from .config import XASCONF, CONF_FILE
+from .config import XASCONF, CONF_FILE,  OLDCONF_FILE
 
 class XASController():
     """
-    class holding the Larch session and doing the processing work for XAS GUI
+    class holding the Larch session and doing the processing work for Larix
     """
     def __init__(self, wxparent=None, _larch=None):
         self.wxparent = wxparent
@@ -30,9 +34,13 @@ class XASController():
         self.plot_erange = None
         self.report_frame = None
         self.recentfiles = []
+        self.panels = {}
+        self.datagroup_callbacks = {}
         self.larch = _larch
         if _larch is None:
             self.larch = larch.Interpreter()
+        self.larix_folder = Path(user_larchdir, 'larix').as_posix()
+        self.config_file = Path(self.larix_folder, CONF_FILE).as_posix()
         self.init_larch_session()
         self.init_workdir()
 
@@ -42,18 +50,27 @@ class XASController():
 
         config = {}
         config.update(XASCONF)
-        xasv_folder = os.path.join(user_larchdir, 'xas_viewer')
-        if not os.path.exists(xasv_folder):
+        # may migrate old 'xas_viewer' folder to 'larix' folder
+        xasv_folder = Path(user_larchdir, 'xas_viewer')
+        if Path(xasv_folder).exists() and not Path(self.larix_folder).exists():
+            print("Migrating xas_viewer to larix folder")
+            shutil.move(xasv_folder, self.larix_folder)
+
+        if not Path(self.larix_folder).exists():
             try:
-                os.mkdir(xasv_folder)
+                mkdir(self.larix_folder)
             except:
-                print("cannot create directory ", xasv_folder)
+                title = "Cannot create Larix folder"
+                message = [f"Cannot create directory {larix_folder}"]
+                ExceptionPopup(self, title, message)
 
-        self.config_file = CONF_FILE
-        if os.path.exists(xasv_folder):
-            self.config_file = os.path.join(xasv_folder, CONF_FILE)
 
-        if os.path.exists(self.config_file):
+        # may migrate old 'xas_viewer.conf' file to 'larix.conf'
+        old_config_file = Path(self.larix_folder, OLDCONF_FILE).as_posix()
+        if Path(old_config_file).exists() and not Path(self.config_file).exists():
+            shutil.move(old_config_file, self.config_file)
+
+        if Path(self.config_file).exists():
             user_config = read_config(self.config_file)
             if user_config is not None:
                 for sname in config:
@@ -65,11 +82,87 @@ class XASController():
                         else:
                             config[sname] = val
 
-        self.config = self.larch.symtable._sys.xasviewer_config = config
+        self.config = self.larch.symtable._sys.larix_config = config
         self.larch.symtable._sys.wx.plotopts = config['plot']
         self.clean_autosave_sessions()
 
 
+    def install_group(self, groupname, filename, source=None, journal=None):
+        """add groupname / filename to list of available data groups"""
+
+        try:
+            thisgroup = getattr(self.symtable, groupname)
+        except AttributeError:
+            thisgroup = self.symtable.new_group(groupname)
+
+        # file /group may already exist in list
+        if filename in self.file_groups:
+            fbase, i = filename, 0
+            while i < 50000 and filename in self.file_groups:
+                filename = f"{fbase}_{i}"
+                i += 1
+                if i >= 50000:
+                    raise ValueError(f"Too many repeated filenames: {fbase}")
+
+        filename = filename.strip()
+        if source is None:
+            source = filename
+
+        jopts = f"source='{source}'"
+        if isinstance(journal, dict):
+            jnl =  {'source': f"{source}"}
+            jnl.update(journal)
+            jopts = ', '.join([f"{k}='{v}'" for k, v in jnl.items()])
+        elif isinstance(journal, (list, Journal)):
+            jopts = repr(journal)
+
+        cmds = [f"{groupname:s}.groupname = {groupname:s}.__name__ = '{groupname:s}'",
+                f"{groupname:s}.filename = '{filename:s}'"]
+        needs_config = not hasattr(thisgroup, 'config')
+        if needs_config:
+            cmds.append(f"{groupname:s}.config = group(__name__='larix config')")
+
+        cmds.append(f"{groupname:s}.journal = journal({jopts:s})")
+
+        if not hasattr(thisgroup, 'xdat'):
+            for xattr in ('x', 'xplot', 'energy', 'xdata'):
+                if hasattr(thisgroup, xattr):
+                    thisgroup.xdat = deepcopy(getattr(thisgroup, xattr))
+                    break
+
+        if not hasattr(thisgroup, 'ydat'):
+            for yattr in ('y', 'yplot', 'mu', 'norm', 'ydata',
+                          'munorm', 'mutrans', 'signal'):
+                if hasattr(thisgroup, yattr):
+                    thisgroup.ydat = deepcopy(getattr(thisgroup, yattr))
+                    break
+
+        if hasattr(thisgroup, 'xdat') and not hasattr(thisgroup, 'xplot'):
+            thisgroup.xplot = deepcopy(thisgroup.xdat)
+        if hasattr(thisgroup, 'ydat') and not hasattr(thisgroup, 'yplot'):
+            thisgroup.yplot = deepcopy(thisgroup.ydat)
+
+        datatype = getattr(thisgroup, 'datatype', 'xydata')
+        if datatype == 'xas':
+            cmds.append(f"{groupname:s}.energy_orig = {groupname:s}.energy[:]")
+            array_labels = getattr(thisgroup, 'array_labels', [])
+            if len(array_labels) > 2  and getattr(thisgroup, 'data', None) is not None:
+                for i0name in ('i0', 'i_0', 'monitor'):
+                    if i0name in array_labels:
+                        i0x = array_labels.index(i0name)
+                        cmds.append(f"{groupname:s}.i0 = {groupname:s}.data[{i0x}, :]")
+
+        self.larch.eval('\n'.join(cmds))
+
+        if needs_config:
+            self.init_group_config(thisgroup)
+
+        self.file_groups[filename] = groupname
+        self.filelist.Append(filename)
+        self.filelist.SetStringSelection(filename)
+        self.sync_xasgroups()
+        return filename
+
     def sync_xasgroups(self):
         "make sure `_xasgroups` is identical to file_groups"
         if self.file_groups != self.symtable._xasgroups:
@@ -84,7 +177,7 @@ class XASController():
     def init_group_config(self, dgroup):
         """set up 'config' group with values from self.config"""
         if not hasattr(dgroup, 'config'):
-            dgroup.config = larch.Group(__name__='xas_viewer config')
+            dgroup.config = larch.Group(__name__='larix config')
 
         for sect in ('exafs', 'feffit', 'lincombo', 'pca', 'prepeaks',
                      'regression', 'xasnorm'):
@@ -93,11 +186,13 @@ class XASController():
     def get_plot_conf(self):
         """get basic plot options to pass to plot() ** not window sizes **"""
         dx = {'linewidth': 3, 'markersize': 4,
-              'show_grid': True, 'show_fullbox': True, 'theme': 'light'}
+              'show_grid': True, 'show_fullbox': True, 'theme': '<Auto>'}
         pconf = self.config['plot']
         out = {}
         for attr, val in dx.items():
             out[attr] = pconf.get(attr, val)
+        if out['theme'].startswith('<Au'):
+            out['theme'] = 'dark' if darkdetect.isDark() else 'light'
         return out
 
     def save_config(self):
@@ -112,34 +207,32 @@ class XASController():
 
     def save_workdir(self):
         """save last workdir and recent session files"""
-        xasv_folder = os.path.join(user_larchdir, 'xas_viewer')
-        if os.path.exists(xasv_folder):
-            try:
-                with open(os.path.join(xasv_folder, 'workdir.txt'), 'w') as fh:
-                    fh.write("%s\n" % get_cwd())
-            except:
-                pass
+        try:
+            with open(Path(self.larix_folder, 'workdir.txt'), 'w') as fh:
+                fh.write(f"{get_cwd()}\n")
+        except:
+            pass
 
-            buffer = []
-            rfiles = []
-            for tstamp, fname in sorted(self.recentfiles, key=lambda x: x[0], reverse=True)[:10]:
-                if fname not in rfiles:
-                    buffer.append(f"{tstamp:.1f} {fname:s}")
-                    rfiles.append(fname)
-            buffer.append('')
-            buffer = '\n'.join(buffer)
+        buffer = []
+        rfiles = []
+        for tstamp, fname in sorted(self.recentfiles, key=lambda x: x[0], reverse=True)[:10]:
+            if fname not in rfiles:
+                buffer.append(f"{tstamp:.1f} {fname:s}")
+                rfiles.append(fname)
+        buffer.append('')
+        buffer = '\n'.join(buffer)
 
-            try:
-                with open(os.path.join(xasv_folder, 'recent_sessions.txt'), 'w') as fh:
-                    fh.write(buffer)
-            except:
-                pass
+        try:
+            with open(Path(self.larix_folder, 'recent_sessions.txt'), 'w') as fh:
+                fh.write(buffer)
+        except:
+            pass
 
     def init_workdir(self):
         """set initial working folder, read recent session files"""
         if self.config['main'].get('use_last_workdir', False):
-            wfile = os.path.join(user_larchdir, 'xas_viewer', 'workdir.txt')
-            if os.path.exists(wfile):
+            wfile = Path(self.larix_folder, 'workdir.txt')
+            if wfile.exists():
                 try:
                     with open(wfile, 'r') as fh:
                         workdir = fh.readlines()[0][:-1]
@@ -151,8 +244,8 @@ class XASController():
             except:
                 pass
 
-        rfile = os.path.join(user_larchdir, 'xas_viewer', 'recent_sessions.txt')
-        if os.path.exists(rfile):
+        rfile = Path(self.larix_folder, 'recent_sessions.txt')
+        if rfile.exists():
             with open(rfile, 'r') as fh:
                 for line in fh.readlines():
                     if len(line) < 2 or line.startswith('#'):
@@ -163,6 +256,26 @@ class XASController():
                     except:
                         pass
 
+    def register_group_callback(self, label, obj, callback):
+        self.datagroup_callbacks[label] = (obj, callback)
+
+    def unregister_group_callback(self, label):
+        if label in self.datagroup_callbacks:
+            self.datagroup_callbacks.pop(label)
+
+    def run_group_callbacks(self):
+        "run callbacks for group changes, checking that objects are still alive"
+        missing = []
+        for key, val in self.datagroup_callbacks.items():
+            try:
+                obj, cb = val
+                obj.Raise()
+                cb()
+            except:
+                missing.append(key)
+        for key in missing:
+            self.unregister_group_callback(key)
+
 
     def autosave_session(self):
         conf = self.get_config('autosave', {})
@@ -170,13 +283,13 @@ class XASController():
         nhistory = max(8, int(conf.get('nhistory', 4)))
 
         fname =  f"{fileroot:s}_{get_sessionid():s}.larix"
-        savefile = os.path.join(user_larchdir, 'xas_viewer', fname)
+        savefile = Path(self.larix_folder, fname).as_posix()
         for i in reversed(range(1, nhistory)):
             curf = savefile.replace('.larix', f'_{i:d}.larix' )
-            if os.path.exists(curf):
+            if Path(curf).exists():
                 newf = savefile.replace('.larix', f'_{i+1:d}.larix' )
                 shutil.move(curf, newf)
-        if os.path.exists(savefile):
+        if Path(savefile).exists():
             curf = savefile.replace('.larix', '_1.larix' )
             shutil.move(savefile, curf)
         save_session(savefile, _larch=self.larch)
@@ -187,11 +300,10 @@ class XASController():
         fileroot = conf.get('fileroot', 'autosave')
         max_hist = int(conf.get('maxfiles', 10))
 
-        xasv_dir = os.path.join(user_larchdir, 'xas_viewer')
         def get_autosavefiles():
             dat = []
-            for afile in os.listdir(xasv_dir):
-                ffile = os.path.join(xasv_dir, afile)
+            for afile in os.listdir(self.larix_folder):
+                ffile = Path(self.larix_folder, afile)
                 if afile.endswith('.larix'):
                     mtime = os.stat(ffile).st_mtime
                     words = afile.replace('.larix', '').split('_')
@@ -200,7 +312,7 @@ class XASController():
                         words.pop()
                     except:
                         version = 0
-                    dat.append((ffile, version, mtime))
+                    dat.append((ffile.as_posix(), version, mtime))
             return sorted(dat, key=lambda x: x[2])
 
         dat = get_autosavefiles()
@@ -230,13 +342,10 @@ class XASController():
         conf = self.get_config('autosave', {})
         fileroot = conf.get('fileroot', 'autosave')
         max_hist = int(conf.get('maxfiles', 10))
-        xasv_dir = os.path.join(user_larchdir, 'xas_viewer')
-
         flist = []
-        for afile in os.listdir(xasv_dir):
-            ffile = os.path.join(xasv_dir, afile)
+        for afile in os.listdir(self.larix_folder):
+            ffile = Path(self.larix_folder, afile).as_posix()
             if ffile.endswith('.larix'):
-                mtime = os.stat(ffile).st_mtime
                 flist.append((os.stat(ffile).st_mtime, ffile))
 
         return sorted(flist, key=lambda x: x[0], reverse=True)[:max_hist]
@@ -247,6 +356,17 @@ class XASController():
         self.filelist.Clear()
         self.init_larch_session()
 
+    def save_exafsplot_config(self, options):
+        self.larix_folder = Path(user_larchdir, 'larix').as_posix()
+        plot_conf = Path(user_larchdir, 'larix', 'larix_exafsplots.conf')
+        save_config(plot_conf, options)
+
+    def load_exafsplot_config(self):
+        plot_conf = Path(user_larchdir, 'larix', 'larix_exafsplots.conf')
+        conf = {}
+        if plot_conf.exists():
+            conf = read_config(plot_conf)
+        return conf
 
     def write_message(self, msg, panel=0):
         """write a message to the Status Bar"""
@@ -256,11 +376,13 @@ class XASController():
         "close all displays, as at exit"
         self.symtable._plotter.close_all_displays()
 
-    def get_display(self, win=1, stacked=False):
+    def get_display(self, win=1, stacked=False,
+                    size=None, position=None):
         wintitle='Larch XAS Plot Window %i' % win
 
-        conf = self.get_config('plot')
-        opts = dict(wintitle=wintitle, stacked=stacked, win=win)
+        conf = self.get_plot_conf()
+        opts = dict(wintitle=wintitle, stacked=stacked,
+                    size=size, position=position, win=win)
         opts.update(conf)
         return self.symtable._plotter.get_display(**opts)
 
@@ -271,12 +393,8 @@ class XASController():
         """
         if topwin is None:
             topwin = wx.GetApp().GetTopWindow()
-            flist = self.filelist
-        else:
-            flist = getattr(topwin, 'filelist', topwin)
-        time.sleep(0.025)
         topwin.Raise()
-        flist.SetFocus()
+        self.filelist.SetFocus()
 
     def get_group(self, groupname=None):
         if groupname is None:
@@ -299,37 +417,37 @@ class XASController():
 
     def merge_groups(self, grouplist, master=None, yarray='mu', outgroup=None):
         """merge groups"""
-        cmd = """%s = merge_groups(%s, master=%s,
-        xarray='energy', yarray='%s', kind='cubic', trim=True)
-        """
-        glist = "[%s]" % (', '.join(grouplist))
+        if master is None:
+            master = grouplist[0]
+        gmaster = self.get_group(master)
+        xarray = 'xplot' if gmaster.datatype=='xydata' else 'energy'
         outgroup = fix_varname(outgroup.lower())
         if outgroup is None:
             outgroup = 'merged'
-
         outgroup = unique_name(outgroup, self.file_groups, max=1000)
 
-        cmd = cmd % (outgroup, glist, master, yarray)
+        glist = "[%s]" % (', '.join(grouplist))
+
+        cmd = f"""{outgroup} = merge_groups({glist}, master={master},
+        xarray='{xarray}', yarray='{yarray}', kind='cubic', trim=True)"""
         self.larch.eval(cmd)
 
-        if master is None:
-            master = grouplist[0]
         this = self.get_group(outgroup)
-        master = self.get_group(master)
-        if not hasattr(master, 'config'):
-            self.init_group_config(master)
+        if not hasattr(gmaster, 'config'):
+            self.init_group_config(gmaster)
         if not hasattr(this, 'config'):
             self.init_group_config(this)
-        this.config.xasnorm.update(master.config.xasnorm)
-        this.datatype = master.datatype
-        this.xdat = 1.0*this.energy
-        this.ydat = 1.0*getattr(this, yarray)
+        this.config.xasnorm.update(gmaster.config.xasnorm)
+        this.datatype = gmaster.datatype
+        if xarray == 'energy':
+            this.xplot = 1.0*this.energy
+        this.yplot = 1.0*getattr(this, yarray)
         this.yerr =  getattr(this, 'd' + yarray, 1.0)
         if yarray != 'mu':
-            this.mu = this.ydat
-        this.plot_xlabel = 'energy'
+            this.mu = this.yplot
+        this.plot_xlabel = xarray
         this.plot_ylabel = yarray
-        return outgroup
+        return this
 
     def set_plot_erange(self, erange):
         self.plot_erange = erange
@@ -346,7 +464,7 @@ class XASController():
         for attr in dir(ogroup):
             val = getattr(ogroup, attr, None)
             if val is not None:
-                setattr(ngroup, attr, copy.deepcopy(val))
+                setattr(ngroup, attr, deepcopy(val))
 
         if new_filename is None:
             new_filename = filename + '_1'
@@ -371,8 +489,11 @@ class XASController():
             plotcmd = newplot
 
         dgroup = self.get_group(groupname)
-        if not hasattr(dgroup, 'xdat'):
-            print("Cannot plot group ", groupname)
+        if not hasattr(dgroup, 'xplot'):
+            if hasattr(dgroup, 'xdat'):
+                dgroup.xplot = deepcopy(dgroup.xdat)
+            else:
+                print("Cannot plot group ", groupname)
 
         if ((getattr(dgroup, 'plot_yarrays', None) is None or
              getattr(dgroup, 'energy', None) is None or
@@ -383,7 +504,7 @@ class XASController():
             plot_yarrays = dgroup.plot_yarrays
 
         popts = kws
-        path, fname = os.path.split(dgroup.filename)
+        fname = Path(dgroup.filename).name
         if not 'label' in popts:
             popts['label'] = dgroup.plot_ylabel
 
@@ -408,7 +529,7 @@ class XASController():
                 popts['label'] = yalabel
             popts['delay_draw'] = (i != narr)
 
-            plotcmd(dgroup.xdat, getattr(dgroup, yaname), **popts)
+            plotcmd(dgroup.xplot, getattr(dgroup, yaname), **popts)
             plotcmd = oplot
 
         if plot_extras is not None:
