@@ -12,30 +12,38 @@ the path represented by the feffNNNN.dat
 
 creates a group that contains the chi(k) for the sum of paths.
 """
-import os
+from pathlib import Path
 import numpy as np
 from copy import deepcopy
 from scipy.interpolate import UnivariateSpline
 from lmfit import Parameters, Parameter
-from lmfit.printfuncs import gformat
 
 from xraydb import atomic_mass, atomic_symbol
+from pyshortcuts import fix_varname
 
-from larch import Group, isNamedClass
-from larch.utils.strutils import fix_varname, b32hash
+from larch.symboltable import Group
+from larch.larchlib import isNamedClass
+from larch.utils.strutils import b32hash
 from larch.fitting import group2params, dict2params, isParameter, param_value
-
-from .xafsutils import ETOK, ktoe, set_xafsGroup
+from .xafsutils import ETOK, ktoe, set_xafsGroup, gfmt, gformat
 from .sigma2_models import add_sigma2funcs
 
 SMALL_ENERGY = 1.e-6
 
+PATH_PARS = ('degen', 's02', 'e0', 'ei', 'deltar', 'sigma2', 'third', 'fourth')
+FDAT_ARRS = ('real_phc', 'mag_feff', 'pha_feff', 'red_fact',
+             'lam', 'rep', 'pha', 'amp', 'k')
+
+# values that will be available in calculations of Path Parameter values
+FEFFDAT_VALUES = ('reff', 'nleg', 'degen', 'rmass', 'rnorman',
+                  'gam_ch', 'rs_int', 'vint', 'vmu', 'vfermi')
+
 class FeffDatFile(Group):
     def __init__(self, filename=None,  **kws):
         kwargs = dict(name='feff.dat: %s' % filename)
         kwargs.update(kws)
         Group.__init__(self,  **kwargs)
-        if filename is not None:
+        if filename not in ('', None) and Path(filename).exists():
             self._read(filename)
 
     def __repr__(self):
@@ -74,6 +82,19 @@ class FeffDatFile(Group):
     @rmass.setter
     def rmass(self, val):  pass
 
+    def _set_from_dict(self, **kws):
+        self.__rmass = None
+        for key, val in kws.items():
+            if key  == 'rmass':
+                continue
+            elif key == 'reff':
+                key = '__reff__'
+            elif key == 'nleg':
+                key = '__nleg__'
+            elif key in FDAT_ARRS:
+                val = np.array(val)
+            setattr(self, key, val)
+
     def __setstate__(self, state):
         (self.filename, self.title, self.version, self.shell,
          self.absorber, self.degen, self.__reff__, self.__nleg__,
@@ -109,7 +130,7 @@ class FeffDatFile(Group):
             with open(filename, 'r') as fh:
                 lines = fh.readlines()
         except:
-            print('Error reading file %s ' % filename)
+            print(f"Error reading Feff Data file '{filename}'")
             return
         self.filename = filename
         mode = 'header'
@@ -193,13 +214,10 @@ class FeffDatFile(Group):
         self.__rmass = None  # reduced mass of path
 
 
-PATH_PARS = ('degen', 's02', 'e0', 'ei', 'deltar', 'sigma2', 'third', 'fourth')
-
 class FeffPathGroup(Group):
-    def __init__(self, filename='', label='', feffrun='', s02=None, degen=None,
+    def __init__(self, filename=None, label='', feffrun='', s02=None, degen=None,
                  e0=None, ei=None, deltar=None, sigma2=None, third=None,
-                 fourth=None, use=True, **kws):
-
+                 fourth=None, use=True, _feffdat=None, **kws):
         kwargs = dict(filename=filename)
         kwargs.update(kws)
         Group.__init__(self, **kwargs)
@@ -213,19 +231,19 @@ class FeffPathGroup(Group):
         self.geom  = []
         self.shell = 'K'
         self.absorber = None
-        self._feffdat = None
-
-        self.hashkey = 'p000'
+        self._feffdat = _feffdat
+        self.dataset = 'd001'
+        self.hashkey = 'p001'
         self.k = None
         self.chi = None
 
         self.__def_degen = 1
-        if filename not in ('', None):
-            if not os.path.exists(filename):
-                raise ValueError(f"Feff Path file '{filename:s}' not found")
-            self._feffdat = FeffDatFile(filename=filename)
-            self.create_spline_coefs()
 
+        if filename not in ('', None) and Path(filename).exists():
+            self._feffdat = FeffDatFile(filename=filename)
+
+        if self._feffdat is not None:
+            self.create_spline_coefs()
             self.geom  = self._feffdat.geom
             self.shell = self._feffdat.shell
             self.absorber = self._feffdat.absorber
@@ -237,9 +255,7 @@ class FeffPathGroup(Group):
 
             if feffrun in ('',  None):
                 try:
-                    dirname, fpfile = os.path.split(filename)
-                    parent, folder = os.path.split(dirname)
-                    self.feffrun = folder
+                    self.feffrun = Path(filename).parent.name
                 except:
                     pass
 
@@ -247,6 +263,7 @@ class FeffPathGroup(Group):
                               deltar=deltar, sigma2=sigma2, third=third,
                               fourth=fourth)
 
+
     def init_path_params(self, degen=None, s02=None, e0=None, ei=None,
                        deltar=None, sigma2=None, third=None, fourth=None):
         """set inital values/expressions for path parameters for Feff Path"""
@@ -283,7 +300,6 @@ class FeffPathGroup(Group):
              self.s02, self.e0, self.ei, self.deltar, self.sigma2,
              self.third, self.fourth, self.use, _feffdat_state) = state
 
-
         self._feffdat = FeffDatFile()
         self._feffdat.__setstate__(_feffdat_state)
 
@@ -306,13 +322,13 @@ class FeffPathGroup(Group):
             rep.extend(atom)
         rep.append("%7.4f" % self._feffdat.reff)
         s = "|".join([str(i) for i in rep])
-        return "p%s" % (b32hash(s)[:9].lower())
+        return "p%s" % (b32hash(s)[:8].lower())
 
     def pathpar_name(self, parname):
         """
         get internal name of lmfit Parameter for a path paramter, using Path's hashkey
         """
-        return f'{parname}_{self.hashkey}'
+        return f'{parname}_{self.dataset}_{self.hashkey}'
 
     def __copy__(self):
         newpath = FeffPathGroup()
@@ -346,15 +362,18 @@ class FeffPathGroup(Group):
     def __repr__(self):
         return f'<FeffPath Group label={self.label:s}, filename={self.filename:s}, use={self.use}>'
 
-    def create_path_params(self, params=None):
+    def create_path_params(self, params=None, dataset=None):
         """
         create Path Parameters within the current lmfit.Parameters namespace
         """
+        if isinstance(params, Group):
+            params = group2params(params)
         if params is not None:
            self.params = params
         if self.params is None:
             self.params = Parameters()
-
+        if dataset is not None:
+            self.dataset = dataset
         if (not isinstance(self.params, Parameters) and
             isinstance(self.params, dict)):
             self.params = dict2params(self.params)
@@ -387,16 +406,16 @@ class FeffPathGroup(Group):
         """stores data about this Feff path in the Parameters
         symbol table for use as `reff` and in sigma2 calcs
         """
+        if self.params is None:
+            self.create_path_params()
         if (not isinstance(self.params, Parameters) and
             isinstance(self.params, dict)):
             self.params = dict2params(self.params)
 
         symtab = self.params._asteval.symtable
         symtab['feffpath'] = self._feffdat
-        symtab['reff'] = self._feffdat.reff
-        symtab['vint'] = self._feffdat.vint
-        symtab['vmu']  = self._feffdat.vmu
-        symtab['vfermi'] = self._feffdat.vfermi
+        for attr in FEFFDAT_VALUES:
+            symtab[attr] = getattr(self._feffdat, attr)
 
 
     def __path_params(self, **kws):
@@ -408,6 +427,7 @@ class FeffPathGroup(Group):
         self.store_feffdat()
         if self.params is None:
             self.create_path_params()
+
         out = []
         for pname in PATH_PARS:
             val = kws.get(pname, None)
@@ -422,6 +442,68 @@ class FeffPathGroup(Group):
         return dict(degen=deg, s02=s02, e0=e0, ei=ei, deltar=delr,
                     sigma2=ss2, third=c3, fourth=c4)
 
+    def dict_report(self):
+        """report as a dict"""
+        tmpvals = self.__path_params()
+        pathpars = {}
+        for pname in ('degen', 's02', 'e0', 'deltar',
+                      'sigma2', 'third', 'fourth', 'ei'):
+            parname = self.pathpar_name(pname)
+            if parname in self.params:
+                pathpars[pname] = (self.params[parname].value,
+                                   self.params[parname].stderr)
+
+        out = {'filename': self.filename,
+                 'label': self.label,
+                 'absorber': self.absorber,
+                 'shell': self.shell,
+                 'nleg': f'{self.nleg}',
+                 'reff': f'{self.reff:.6f}'}
+
+        geom = []
+        gshort = []
+        geomformat = '%s(ipot=%d): %s, %s, %s'
+        for atsym, iz, ipot, amass, x, y, z in self.geom:
+            geom.append(geomformat % (atsym, ipot, x, y, z))
+            gshort.append(f'{atsym}({ipot})')
+        out['geometry'] = '\n'.join(geom)
+        out['geom'] = '-'.join(gshort)
+
+        def sfmt(x): return gformat(x, length=8)
+
+        for pname in ('degen', 's02', 'e0', 'r',
+                      'deltar', 'sigma2', 'third', 'fourth', 'ei'):
+            val = strval = getattr(self, pname, 0)
+            parname = self.pathpar_name(pname)
+            std, expr = None, None
+            if pname == 'r':
+                parname = self.pathpar_name('deltar')
+                par = self.params.get(parname, None)
+                val = par.value + self._feffdat.reff
+                strval = 'reff + ' + getattr(self, 'deltar', 0)
+                std = par.stderr
+            else:
+                if pname in pathpars:
+                    val, std = pathpars[pname]
+                else:
+                    par = self.params.get(parname, None)
+                    if par is not None:
+                        val = par.value
+                        std = par.stderr
+            if std is None  or std <= 0:
+                svalue = sfmt(val)
+                std = 0.0
+            else:
+                svalue = f"{sfmt(val)}({sfmt(std).strip()})"
+            out[pname] = svalue
+            out[pname+'_value'] = sfmt(val)
+            if abs(std) <  1.e-9:
+                out[pname+'_stderr'] = '0.0'
+            else:
+                out[pname+'_stderr'] = sfmt(std)
+            out[pname+'_expr'] = strval
+        return out
+
     def report(self):
         "return  text report of parameters"
         tmpvals = self.__path_params()
@@ -446,7 +528,7 @@ class FeffPathGroup(Group):
 
         stderrs = {}
         out.append('     {:7s}= {:s}'.format('reff',
-                                              gformat(self._feffdat.reff)))
+                                              gfmt(self._feffdat.reff)))
 
         for pname in ('degen', 's02', 'e0', 'r',
                       'deltar', 'sigma2', 'third', 'fourth', 'ei'):
@@ -469,9 +551,9 @@ class FeffPathGroup(Group):
                         std = par.stderr
 
             if std is None  or std <= 0:
-                svalue = gformat(val)
+                svalue = gfmt(val)
             else:
-                svalue = "{:s} +/-{:s}".format(gformat(val), gformat(std))
+                svalue = "{:s} +/-{:s}".format(gfmt(val), gfmt(std))
             if pname == 's02':
                 pname = 'n*s02'
 
@@ -517,8 +599,8 @@ class FeffPathGroup(Group):
         # get values for all the path parameters
         (degen, s02, e0, ei, deltar, sigma2, third, fourth)  = \
                 self.__path_params(degen=degen, s02=s02, e0=e0, ei=ei,
-                                 deltar=deltar, sigma2=sigma2,
-                                 third=third, fourth=fourth)
+                                   deltar=deltar, sigma2=sigma2,
+                                   third=third, fourth=fourth)
 
         # create e0-shifted energy and k, careful to look for |e0| ~= 0.
         en = k*k - e0*ETOK
@@ -568,7 +650,7 @@ class FeffPathGroup(Group):
 
 
 
-def path2chi(path, paramgroup=None, **kws):
+def path2chi(path, params=None, paramgroup=None, **kws):
     """calculate chi(k) for a Feff Path,
     optionally setting path parameter values
     output chi array will be written to path group
@@ -589,17 +671,20 @@ def path2chi(path, paramgroup=None, **kws):
     if not isNamedClass(path, FeffPathGroup):
         msg('%s is not a valid Feff Path' % path)
         return
-    path.calc_chi_from_params(paramgroup, **kws)
+    if params is None and paramgroup is not None:
+        params = group2params(paramgroup)
+    path.calc_chi_from_params(params=params, **kws)
 
 
-def ff2chi(paths, group=None, paramgroup=None, k=None, kmax=None,
-            kstep=0.05,  **kws):
+def ff2chi(paths, group=None, params=None, k=None, kmax=None, kstep=0.05,
+           paramgroup=None, **kws):
     """sum chi(k) for a list of FeffPath Groups.
 
     Parameters:
     ------------
       paths:       a list of FeffPath Groups or dict of {label: FeffPathGroups}
-      paramgroup:  a Parameter Group for calculating Path Parameters [None]
+      params:      lmfit.Parameters for calculating Path Parameters [None]
+      paramgroup:  (old) Group of Parameters for calculating Path Parameters [None]
       kmax:        maximum k value for chi calculation [20].
       kstep:       step in k value for chi calculation [0.05].
       k:           explicit array of k values to calculate chi.
@@ -611,11 +696,11 @@ def ff2chi(paths, group=None, paramgroup=None, k=None, kmax=None,
     `paths` and writes the resulting arrays to group.k and group.chi.
 
     """
-    if isinstance(paramgroup, Parameters):
-        params = paramgroup
-    else:
-        params = group2params(paramgroup)
-
+    if params is None:
+        if isinstance(paramgroup, Parameters):
+            params = paramgroup
+        else:
+            params = group2params(paramgroup)
 
     if isinstance(paths, (list, tuple)):
         pathlist = paths
@@ -630,11 +715,11 @@ def ff2chi(paths, group=None, paramgroup=None, k=None, kmax=None,
 
     for path in pathlist:
         if not isNamedClass(path, FeffPathGroup):
-            print('%s is not a valid Feff Path' % path)
+            print(f"{path} is not a valid Feff Path")
             return
         path.create_path_params(params=params)
         path._calc_chi(k=k, kstep=kstep, kmax=kmax)
-    k = pathlist[0].k[:]
+    k = pathlist[0].k[:]*1.0
     out = np.zeros_like(k)
     for path in pathlist:
         out += path.chi
@@ -672,6 +757,8 @@ def feffpath(filename='', label='', feffrun='', s02=None, degen=None,
     ---------
         a FeffPath Group.
     """
+    if filename != '' and not Path(filename).exists():
+        raise ValueError(f"Feff Path file '{filename:s}' not found")
     return FeffPathGroup(filename=filename, label=label, feffrun=feffrun,
                          s02=s02, degen=degen, e0=e0, ei=ei, deltar=deltar,
                          sigma2=sigma2, third=third, fourth=fourth, use=use)
